@@ -51,9 +51,11 @@ class Session:
     tls_mode: TLSMode = TLSMode.ENABLED
     ssl_context: ssl.SSLContext = None
     authenticator: collections.abc.Callable = None
-    poll: float = 0
+    poll: float = 60
+    idle: float = 900
     mailbox: str = "inbox"
     policy: collections.abc.Callable = None
+    parameters: collections.abc.Mapping = None
 
     @tenacity.retry(
             wait = tenacity.wait_exponential(max = 300),
@@ -85,52 +87,59 @@ class Session:
         has_move = b"MOVE" in capabilities
 
         # choose wait mechanism
-        wait = self._wait_idle if has_idle and self.poll <= 0\
-            else self._wait_poll
+        wait = self._wait_idle if has_idle else self._wait_poll
 
         # process unseen messages
         client.select_folder(self.mailbox, readonly = True)
-        msgids = client.search("UNSEEN")
-        for msgid in msgids:
-            self._process(client, msgid)
+        messages = client.search(["UNSEEN"])
+        for message in messages:
+            self._process(client, message)
 
         # process incoming messages
-        nextid = msgids[-1] + 1 if msgids else 1
+        next_message = max(messages) + 1 if messages else 1
         while True:
             client.select_folder(self.mailbox, readonly = True)
             wait(client)
-            msgids = client.search("{}:* UNSEEN".format(nextid))
-            for msgid in msgids:
-                self._process(client, msgid)
-            nextid = msgids[-1] + 1 if msgids else nextid
+            messages = client.search(["{}:*", "UNSEEN"].format(next_message))
+            for message in messages:
+                self._process(client, message)
+            if messages:
+                next_message = max(messages) + 1
 
-    def _process(self, client, msgid):
+    def _process(self, client, message):
         if self.policy:
             namespace = {
                 "client": client,
                 "mailbox": self.mailbox,
-                "msgid": msgid
+                "message": message,
+                "parameters": dict(self.parameters) if self.parameters else {}
             }
             exec(self.policy, namespace)
 
     def _wait_poll(self, client):
-        poll = self.poll if self.poll > 0 else 60
         while True:
             response = client.noop()
             if response:
                 if any([x for x in response[1] if x[1] == b"EXISTS"]):
-                    return response[1]
+                    return
             else:
                 raise ConnectionError("connection dropped")
-            time.sleep(poll)
+
+            time.sleep(self.poll)
 
     def _wait_idle(self, client):
+        now = time.time()
+        alarm = now + self.idle
         client.idle()
         while True:
-            response = client.idle_check()
+            response = client.idle_check(alarm - now)
             if response:
                 if any([x for x in response if x[1] == b"EXISTS"]):
                     client.idle_done()
-                    return response[1:]
-            else:
-                raise ConnectionError("connection dropped")
+                    return
+
+            now = time.time()
+            if now >= alarm:
+                alarm = now + self.idle
+                client.idle_done()
+                client.idle()
